// ==============================================================
//                 ORBITER MODULE: GLIDER
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// GLIDER.cpp
// Control module for GLIDER vessel class
//

// ==============================================================
#define ORBITER_MODULE
#include "orbitersdk.h"
#include "ARROWJET.h"
//
#include "OrbiterSoundSDK35.h"
VISHANDLE MainExternalMeshVisual = 0;


// 1. vertical lift component (wings and body)

void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{

int i;
	const int nlift = 33;
	static const double AOA[nlift] = {-180*RAD,-165*RAD,-150*RAD,-135*RAD,-120*RAD,-105*RAD,-90*RAD,-75*RAD,-60*RAD,-45*RAD,-30*RAD, -25*RAD , -20*RAD , -15*RAD , -10*RAD , -5*RAD , 0*RAD , 5*RAD , 10*RAD , 15*RAD , 20*RAD , 25*RAD,  30*RAD, 60*RAD, 90*RAD, 120*RAD, 150*RAD, 165*RAD, 180*RAD };
	static const double CL[nlift]  = { -0.131 ,  -0.665,  -0.1  , -0.032 , -0.015 , -0.011 , -0.009, -0.009, -0.011, -0.02 , -0.05, 	-0.08   , -0.13   , -0.24   , -0.4    , -0.32  , -0.13 , 0.56  , 0.82   , 0.54   , 0.26   , 0.14 , 	0.081 , 0.013 , 0.008 , 0.013  , 0.062  , 0.295  , -0.132   };
	static const double CD[nlift]  = { 0.01358,0.13833 , 0.846  ,1.307   ,1.73    , 2.49   , 2.61  , 2.81  , 2.18  , 1.66  , 1.003 , 0.68    , 0.36    , 0.29    , 0.12    , 0.05   , 0.01  , 0.04  , 0.09   , 0.23   , 0.48   , 0.68 , 1.11  , 2.31  , 3.44  , 1.62   , 0.74   , 0.204  , 0.014  };
	static const double CM[nlift]  = { -0.015 , -0.008 , -0.018 , -0.027 , -0.030 , -0.033 , -0.031, -0.025, -0.017, -0.011, -0.005 , -0.00  , 0.03  , 0.003  , 0.001  , -0.00 , -0.00 , -0.001, -0.004, -0.008 , -0.011 , -0.014,  -0.018,-0.030 , -0.031,-0.017  , -0.005 , -0.003 , -0.015 };   	
	double realaoa;
	realaoa = aoa;
	aoa-=1.0*RAD;
	
	
	
	for (i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.015 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (VESSEL *v, double beta, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}


char *GLIDER::SendHudMessage()
{
dHudMessageDelay=15;
return cUmmuHudDisplay;
}



void GLIDER::clbkPostCreation (void)

{


MOONBASEALPHA2ID=ConnectToOrbiterSoundDLL3(GetHandle());
ReplaceStockSound3(MOONBASEALPHA2ID,"Sound\\Vessel\\MOONBASEALPHA\\ambient.wav",	REPLACE_COCKPIT_AMBIENCE_1);
SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYHOVERTHRUST,FALSE);
SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYATTITUDETHRUST,TRUE);

SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYCABINAIRCONDITIONING,TRUE);

SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYCABINRANDOMAMBIANCE,TRUE);

SoundOptionOnOff3(MOONBASEALPHA2ID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MOONBASEALPHA2ID,DISPLAYTIMER,FALSE);

}
void GLIDER::SelectCockpitView (int CAM)
{

     switch (CAM) {
 case 0:  //PILOT
          SetCameraDefaultDirection (_V(0,0,1));
          SetCameraOffset (_V(0,.7,5.4));
		  oapiCameraSetCockpitDir (0,0);
          break;
 case 1:  //PILOT
          SetCameraDefaultDirection (_V(0,0,1));
          SetCameraOffset (_V(0,.7,3.8));
		  oapiCameraSetCockpitDir (0,0);
          break;
 

 }}


// Constructor
GLIDER::GLIDER (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)



{
	tex = oapiRegisterExhaustTexture("turbo_exh");
	//SURFHANDLE tex_main = oapiRegisterExhaustTexture ("turbo_exh");

GEAR_status      = GEAR_UP;
GEAR_proc        = 0.0;

CANOPY1_status      = GEAR_UP;
CANOPY1_proc        = 0.0;

CANOPY2_status      = GEAR_UP;
CANOPY2_proc        = 0.0;
speedbrake=0;

FLAP_proc=0;
LIFT_SPEED =.25;


CAM=0;
DefineAnimations ();

	
}



//
void GLIDER::clbkSetClassCaps (FILEHANDLE cfg)
//void GLIDER::SetTransporter()
{
	// physical specs
	SetSize (11);
	SetEmptyMass (22245);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetCrossSections (_V(50.49,126.07,9.59));
	SetRotDrag (_V(0.1,0.1,0.1));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(33.6,39.94,10.01));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,.7,5.4));
	
	SetTouchdownPoints  (_V(0,-3.31,5), _V(-1.6,-2.72,-5), _V(1.6,-2.72,-5));; 
EnableTransponder (true);

/*hwing = CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,-0.3), VLiftCoeff, 0, 5, 90, 1.5);
	// wing and body lift+drag components

	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 0, 5, 15, 1.5);
	// vertical stabiliser and body lift and drag components

	CreateControlSurface (AIRCTRL_ELEVATOR,     1.4, 1.5, _V(   0,0,-7.2), AIRCTRL_AXIS_XPOS, anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       0.8, 1.5, _V(   0,0,-7.2), AIRCTRL_AXIS_YPOS, anim_rudder);
	hlaileron = CreateControlSurface2 (AIRCTRL_AILERON, 0.3, 1.5, _V( 7.5,0,-7.2), AIRCTRL_AXIS_XPOS, anim_raileron);
	hraileron = CreateControlSurface2 (AIRCTRL_AILERON, 0.3, 1.5, _V(-7.5,0,-7.2), AIRCTRL_AXIS_XNEG, anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 0.3, 1.5, _V(   0,0,-7.2), AIRCTRL_AXIS_XPOS, anim_elevatortrim);

*/
		CreateAirfoil3 (LIFT_VERTICAL, _V(0,0.2,0), VLiftCoeff, 0,13.6, 113, 2.04);
	// wing and body lift+drag components

	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,-17), HLiftCoeff, 0,3, 25, 1.0);
	// vertical stabiliser and body lift and drag components

	CreateControlSurface (AIRCTRL_ELEVATOR,     10.0, 0.4, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_ELEVATOR);
	CreateControlSurface (AIRCTRL_RUDDER,       5.0, 0.2, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_RUDDER);
	CreateControlSurface (AIRCTRL_AILERON,      10, 0.5, _V( 15,0,-15), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      10, 0.5, _V(-15,0,-15), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 2, 0.3, _V(   0,0,-15), AIRCTRL_AXIS_XPOS);
CreateControlSurface (AIRCTRL_FLAP,         24, 1.5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);

	CreateVariableDragElement( &speedbrake, 8.0, _V(0,0,0) );

	CreateVariableDragElement (&GEAR_proc, 0.8, _V(0, -1, 0));

	
//BEACON
static VECTOR3 APbeaconcol[3] = {{1,0,0},
                                    {0, 1, 0},
									{1, 1, 1} };
                               

    for ( j = 0; j < 3; j++) {
        APbeacon[j].shape = BEACONSHAPE_STAR;
        APbeacon[j].pos = APbeaconpos+j;
        APbeacon[j].col = APbeaconcol+j;
        APbeacon[j].size = .25;
        APbeacon[j].falloff =  0.4;
        APbeacon[j].period = 0.8;
        APbeacon[j].duration = 0.3;
        APbeacon[j].tofs = (3-j)*0.2;
        APbeacon[j].active = true;
        AddBeacon (APbeacon+j);
    }

        APbeacon[0].active = true;
   
APbeaconpos[0] = (_V(-7.662051, .183, -9.146784));
APbeaconpos[1] = (_V(7.662051, .183, -9.146784));
APbeaconpos[2] = (_V(0,4.24, -10.67332));




// visual specs
MAIN = AddMesh (oapiLoadMeshGlobal ("AARROW1"));
SetMeshVisibilityMode (MAIN, MESHVIS_ALWAYS);

MAINVC = AddMesh (oapiLoadMeshGlobal ("AARROW1A"));
SetMeshVisibilityMode (MAINVC, MESHVIS_VC);





// propellant resources

    PROPELLANT_HANDLE TANK,hpr;
    TANK = CreatePropellantResource (FUELMASS);
    hpr = CreatePropellantResource (0);

	// ***************** thruster definitions *******************
	

	//th_main = CreateThruster (_V(0,0,-10), _V(0,0,1), MAXMAINTH, TANK, ISP);
	//CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);


// Call clbkSetClassCaps_UMMU and select a member
	SelectedUmmuMember=0;
	clbkSetClassCaps_UMMu();

	
 PARTICLESTREAMSPEC contrail = 
	{
		0, 1.5, 1000, 100, 0.01, 5.0, 0.3, 1.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PLIN, 0.3, 1.8,
		PARTICLESTREAMSPEC::ATM_PLIN, 0.652, 0.272
	};

/*	PARTICLESTREAMSPEC exhaust_main = {
		0, 1, 1000, 200, 0.01, 0.5, 0.9, 1.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PLIN, 0.8, 1.8,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-10, 1e-10
	};
*/	PARTICLESTREAMSPEC exhaust_main = {
		0, 1, 1000, 100, 0.01, 0.5, 0.9, 1.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PLIN, 0.8, 1.8,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-10, 1e-10
	};
    exhaust_main.tex = oapiRegisterParticleTexture ("Contrail");

	th_first[0] = CreateThruster (_V(-.749,0,-10.7), _V(0,0,1), MAXMAINTH, TANK, ISP);
	th_first[1] = CreateThruster (_V(.749,0,-10.7), _V(0,0,1), MAXMAINTH, TANK, ISP);

	th_AB[0] = CreateThruster (_V(-.749,0,-10.7), _V(0,0,1), MAXMAINTHAB, TANK, ISPAB);
	th_AB[1] = CreateThruster (_V(.749,0,-10.7), _V(0,0,1), MAXMAINTHAB, TANK, ISPAB);



	AddExhaust( th_first[0], 4.33, 0.4,tex );
    AddExhaust( th_first[1], 4.33, 0.4,tex );
	

	AddExhaust( th_AB[0], 10, 0.6,tex );
    AddExhaust( th_AB[1], 10, 0.6,tex );


	AddExhaustStream( th_first[0], &exhaust_main );
	AddExhaustStream( th_first[1], &exhaust_main );

	AddExhaustStream( th_first[0], _V(-0.763,-0.464,-10.7), &contrail );
	AddExhaustStream( th_first[1], _V(0.763,-0.464,-10.7), &contrail );
  
	CreateThrusterGroup( th_first, 2, THGROUP_MAIN );


//FAKETHRUSTERS
	
	th_att_rot[0] = CreateThruster (_V(0.0, 0, 1), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[1] = CreateThruster (_V(0.0, 0, -1), _V(0,1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[2] = CreateThruster (_V(0.0, 0, 1), _V(0,1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[3] = CreateThruster (_V(0.0, 0, -1), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHDOWN);
	CreateThrusterGroup (th_att_rot+2,   2, THGROUP_ATT_PITCHUP);

	th_att_rot[0] = CreateThruster (_V( 1,0,0), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[1] = CreateThruster (_V(-1,0,0), _V(0,1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[2] = CreateThruster (_V(1,0,0), _V(0,1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[3] = CreateThruster (_V(-1,0,0), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	CreateThrusterGroup (th_att_rot, 2, THGROUP_ATT_BANKRIGHT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKLEFT);
	
	th_att_rot[0] = CreateThruster (_V( 14.6,0,0), _V(0,0,-1), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[1] = CreateThruster (_V(-14.6,0,0), _V(0,0,1), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[2] = CreateThruster (_V( 14.6,0,0), _V(0,0,1), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_rot[3] = CreateThruster (_V(-14.6,0,0), _V(0,0,-1), PB_MAXRCSTH, hpr, PB_ISP);
	CreateThrusterGroup (th_att_rot, 2, THGROUP_ATT_YAWRIGHT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWLEFT);


	th_att_lin[0] = CreateThruster (_V( 1,0,0), _V(0,1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[1] = CreateThruster (_V(-1,0,0), _V(0,1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[2] = CreateThruster (_V(1,0,0), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[3] = CreateThruster (_V(-1,0,0), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);	
		
	
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);



	th_att_lin[0] = CreateThruster (_V( 1,0,0), _V(0,0,1), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[1] = CreateThruster (_V(-1,0,0), _V(0,0,1), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[2] = CreateThruster (_V(1,0,0), _V(0,0,-1), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[3] = CreateThruster (_V(-1,0,0), _V(0,0,-1), PB_MAXRCSTH, hpr, PB_ISP);	

	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_BACK);


	th_att_lin[0] = CreateThruster (_V(0,0,1), _V(1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[1] = CreateThruster (_V(0,0,-1), _V(1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[2] = CreateThruster (_V(0,0,1), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_att_lin[3] = CreateThruster (_V(0,0,-1), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);	
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_RIGHT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_LEFT);





}

// =======================================================
// UMMU Class Caps
// =======================================================

void GLIDER::clbkSetClassCaps_UMMu(void)
{
	//--------------------------------------------------------------
	// InitUmmu
	// initialisation of Ummu must be done first ! return of 1 mean OK 
	// return of -999 mean Ummu addon is not installed in user's Orbiter, return of -1 
	// mean misc error. If error simply all function will fail silently 
	// and there will be no MMu. It's a good idea to warn the user. 
	// (see function "WarnUserUMMUNotInstalled" below)
	Crew.InitUmmu(GetHandle());	
	//--------------------------------------------------------------

	//--------------------------------------------------------------
	// GetUserUMmuVersion
	// this check wich version of UMmu addon the user have in his orbiter directory.
	// this may be usefull for future version with more function so you can check
	// if user installation support the new functions. Return -1 on error (not installed, version cannot be read)
	// or version number in float value (ie: 2.0="2.0" version etc etc)
	float UMmuVersion=Crew.GetUserUMmuVersion();

	//--------------------------------------------------------------
	// DefineAirLockShape
	// We set the airlock shape and state, this virtual box will be the one
	// into were a Ummu asking to reenter will be taken in account
	// first parameter is airlock door state (OPEN=TRUE) and other are X,X1,Y,Y1 and Z,Z1 
	// min and max coordinate for this virtual box (vessel local coordinate)
	// Pay attention to reenter when landed (if suitable) and make the box large enough so 
	// it's not a pain for the user to find airlock's entry. (See PDF doc)
	Crew.DefineAirLockShape(TRUE,-7,-1,-5,3,-10,0);			// Airlock open, 2 meter large 4 meter high 6 meter long (vessel local coordinate)

	//--------------------------------------------------------------
	// SetMembersPositionOrientationOnEVA
	// We define here were the Ummu will appear when they make an EVA
	// first parameter is position second is orientation, take care not to make
	// them appear to high on ground (will fall and die) or to low (Orbiter "light speed" bug)
	Crew.SetMembersPosRotOnEVA(_V( -3.5 ,.124,-2.76),_V(0,-1.5,0));	// 3 meters in front of ship (z) facing direction of ship (vessel local coordinate)

	//---------------------------------------------------------------
	// finally set the maximum seat available in this ship, default is
	// 8 but it can go from 1 to 100
	// BAD IDEA: using this to "close" your ship, use "SetAirlockDoorState" instead.
	Crew.SetMaxSeatAvailableInShip(2);

	//----------------------------------------------------------------
	// OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL
	// DeclareActionArea
	// Ever wanted to be able to open a door, repair a system or trigger something in your vessel from an UMMU ?
	// Here come a new feature of UMMU 2.0: Action area. See UMmuSDK.h and PDF doc for complete explanation and see
	// "DetectActionAreaActivated" below.
	//Crew.DeclareActionArea(0,_V(-28,0,32.8),10,TRUE,"action_door.wav","Open/Close Travel Tube Doors");
	//Crew.DeclareActionArea(1,_V(54,0,0),5,TRUE, "action_door.wav","Open/Close Travel Tube Doors");
	//Crew.DeclareActionArea(2,_V(61,0,-11.75),2,TRUE, "action_door.wav","Inner AIrlock Active");
	//Crew.DeclareActionArea(3,_V(65.5,0,-11.75),2,TRUE, "action_door.wav","Outer AIrlock Active");

	//Crew.DeclareActionArea(3,_V(0,0,-5),2.5,TRUE,"action_activated.wav","You are behind the ship");
	//iActionAreaDemoStep=0;	// this is just to show a feature of action area, see below "DetectActionAreaActivated"

	//--------------------------------------------------------------
	// AddCrewMember
	// We'll add four default members for when the ship is spawned by Orbiter's scenario editor. 
	// parameters are name, age, cardiac pulse,weight (in kilogramm) and Function (Misc ID)
	// max four characters wich define the spacesuit used. (see UmmuMiscID in UMuSDK.h header)
	// Return 1 on success -1 on error (probably vessel full, name>25 char 
	// age, pulse or weight out of realistic range or MiscID>4 char)
	Crew.AddCrewMember("Paul Roper",41,65,74,"99M");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Frank Craig",27,67,55,"99");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Craig Collins",25,70,45,"99P");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Doug Jackson",45,70,45,"99D" );	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Keith Ford",45,70,45,"99T");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Paul Foster",45,70,45,"99R" );	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Keith Roper",45,70,45,"99T");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Paul Collins",45,70,45,"99R" );	//(for name and id a-z A-Z 0-9 characters only)
	SelectedUmmuMember =0;  // our current selected member


	// The HUD display method variables, see PDF doc
	cUmmuHudDisplay[0] =0;	// Initialisation of UMmu hud char variable
	dHudMessageDelay =0;	// Initialisation of UMmu delay variable
	strcpy(SendHudMessage(),"Welcome aboard ! E=EVA 1,2=select UMmu D=Open/Close airlock S=info M=add crew");

	// The Add mmu without scenery editor variable see PDF doc
	cAddUMmuToVessel[0]=0;
}








void GLIDER::clbkPostStep(double simt, double simdt, double mjd)
{
//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	// Here the routine that detect if someone entered the ship (eva or transfer)
	// No need to manage anything here all is automatic, crew is added to your ship, ship's weight is updated,
	// and UMmu vessel is automatically deleted from Orbiter. You may just look the return
	// code to see if someone entered and display wathewer message on panel or other.
	// notice it's FPS friendly, function process only 4 time per second not each frame 
	// and return immediately if airlock closed or no Ummu is detected in vincinity. 
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf(SendHudMessage(),"%s \"%s\" aged %i was transfered to our ship",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
		,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf(SendHudMessage(),"%s \"%s\" aged %i entered into our ship",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
		Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}
	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		//int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			//for(I=0;I<Crew.GetCrewTotalNumber();I++)
			//{
		//		Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
		//	}
		//	strcpy(SendHudMessage(),"Oooh no ! Crash - All crew aboard killed");
		}

		// TIPS: the vertical speed is often reset to zero when there is ground contact
		// this may bug somewhat the death of your crew.
		// to have an accurate VertSpeed at touchdown I recommand to record it at very END 
		// of timestep and use this "old" value. The next frame you'll have the vertspeed value 
		// of *last frame* just before the crash (GroundContact). This ensure an accurate 
		// verticalspeed value. (keep this value in your vessel class and don't forget to 
		// initialize it at zero in setclasscap)
	}

//sprintf(oapiDebugString(),"anim %2.2f", DOOR_proc);

	//---------------------------------------------------------------------------
	// WarnUserUMMUNotInstalled - IMPORTANT helper
	// Put here this function and users will be automatically warned if they don't have
	// UMMU installed or if it's outdated. Warning text duration is 20 seconds and text is:
	// [AddonName] require "Universal MMU" ver 2.0 or higher. Download at www.orbiter.dansteph.com (message countdown in seconds)
	// an additonal "your version of UMMU is outdated" is displayed the last 5 seconds if they have version lower than 2.0
	Crew.WarnUserUMMUNotInstalled("GLIDER");



	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	AddUMmuToVessel();




if (GEAR_status >= GEAR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (GEAR_status == GEAR_RAISING) {
			if (GEAR_proc > 0.0) GEAR_proc = max (0.0, GEAR_proc-da);
			else                GEAR_status = GEAR_UP;
		} else {
			if (GEAR_proc < 1.0) GEAR_proc = min (1.0, GEAR_proc+da);
			else                GEAR_status = GEAR_DOWN;
		}

//sprintf(oapiDebugString(),"GEAR %2.2f ",GEAR_proc);

SetAnimation (anim_GEAR, GEAR_proc);		
//SetAnimation (anim_GEAR1B, GEAR_proc);
//SetAnimation (anim_GEAR2A, GEAR_proc);
//SetAnimation (anim_GEAR2B, GEAR_proc);
//SetAnimation (anim_GEAR3A, GEAR_proc);
//SetAnimation (anim_GEAR3B, GEAR_proc);
//SetAnimation (anim_GEAR3C, GEAR_proc);
}		

if (CANOPY1_status >= GEAR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (CANOPY1_status == GEAR_RAISING) {
			if (CANOPY1_proc > 0.0) CANOPY1_proc = max (0.0, CANOPY1_proc-da);
			else                CANOPY1_status = GEAR_UP;
		} else {
			if (CANOPY1_proc < 1.0) CANOPY1_proc = min (1.0, CANOPY1_proc+da);
			else                CANOPY1_status = GEAR_DOWN;
		}


SetAnimation (anim_CANOPY1, CANOPY1_proc);		


}
if (CANOPY2_status >= GEAR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (CANOPY2_status == GEAR_RAISING) {
			if (CANOPY2_proc > 0.0) CANOPY2_proc = max (0.0, CANOPY2_proc-da);
			else                CANOPY2_status = GEAR_UP;
		} else {
			if (CANOPY2_proc < 1.0) CANOPY2_proc = min (1.0, CANOPY2_proc+da);
			else                CANOPY2_status = GEAR_DOWN;
		}


SetAnimation (anim_CANOPY2, CANOPY2_proc);		


}

SetTouchdownPoints (_V(0,-1.44-gear_pos*1.87,5), _V(-1.6,-.97-gear_pos*1.75,-5), _V(1.6,-.97-gear_pos*1.75,-5));

gear_pos = max(0.0, (GEAR_proc-0.5) *2.0);
THROTTLE1=(GetThrusterLevel (th_first[0]));
	 SetAnimation (anim_THROTTLE, THROTTLE1);

	 SetAnimation (anim_LGEAR, GEAR_proc);



	 if (GetThrusterLevel (th_first[0])>.95){
		SetThrusterLevel(th_AB[0],1);
		SetThrusterLevel(th_AB[1],1);}

	 if (GetThrusterLevel (th_first[0])<.95){
		SetThrusterLevel(th_AB[0],0);
		SetThrusterLevel(th_AB[1],0);}



STICK1Z=GetThrusterGroupLevel (THGROUP_ATT_PITCHUP);
STICK1Z1=GetThrusterGroupLevel (THGROUP_ATT_PITCHDOWN);

if (STICK1Z==1)STICK1Z2 =1;
if (STICK1Z1==1)STICK1Z2 =0;
if ((STICK1Z==0)&& (STICK1Z1==0))STICK1Z2 =.5;

SetAnimation (anim_STICKZ, STICK1Z2);
//sprintf(oapiDebugString(),"anim %2.2f", STICK1Z);






STICK1X=GetThrusterGroupLevel (THGROUP_ATT_BANKRIGHT);
STICK1X1=GetThrusterGroupLevel (THGROUP_ATT_BANKLEFT);

if (STICK1X==1)STICK1X2 =1;
if (STICK1X1==1)STICK1X2 =0;
if ((STICK1X==0)&& (STICK1X1==0))STICK1X2 =.5;


SetAnimation (anim_STICKX, STICK1X2);
SetAnimation (anim_SPEEDBRAKE, speedbrake);
//SetAnimation (anim_flaps, FLAP_proc);		

ATMD1= GetAtmDensity () ;

ATMadjusted=(55600*ATMD1);
ATMABadjusted=(48930*ATMD1);

//SetThrusterMax0 (th_first[0],ATMadjusted);
//SetThrusterMax0 (th_first[1],ATMadjusted);
//SetThrusterMax0 (th_AB[0],ATMABadjusted);
//SetThrusterMax0 (th_AB[1],ATMABadjusted);

//SetThrusterMax0 (hover_th, double maxth0) ATMadjusted

//sprintf(oapiDebugString(),"ATM %2.2f ATMADJU %2.2f ATM2 %2.2f ATM3 %2.2f", ATMD1,ATMadjusted,ATMD2,ATMD3);
//ATMD2=((ATMD1-.7)*1.5);
//ATMD3=(ATMD2*8.5e5);
//SetThrusterMax0 (th_first[0],ATMD3);
//SetThrusterMax0 (th_first[1],ATMD3);


PEDALR=GetThrusterGroupLevel (THGROUP_ATT_YAWRIGHT);
PEDALL=GetThrusterGroupLevel (THGROUP_ATT_YAWLEFT);
if (PEDALR==1)PEDALBOTH =1;
if (PEDALL==1)PEDALBOTH =0;
if ((PEDALR==0)&& (PEDALL==0))PEDALBOTH =.5;

 SetAnimation (anim_PEDAL1, PEDALBOTH);
 SetAnimation (anim_PEDAL2, PEDALBOTH);

}

	    
  










void GLIDER::DefineAnimations(void)

{

static UINT WGrp5[1] = {27};
   static MGROUP_ROTATE rudder (0,WGrp5, 1, _V(0,.881, -8.22), _V(0,1,-.9), (float)(-50*RAD));
 anim_RUDDER = CreateAnimation (0.5);
   //   static MGROUP_ROTATE rudder (0,WGrp5, 1, _V(-1.5, 1.5, -6.856527), _V(-.75,.6,.21), (float)(-60*RAD));

   AddAnimationComponent  (anim_RUDDER, 0, 1, &rudder);

   


static UINT WGrp7[1] = {21};//leftelevator
   static MGROUP_ROTATE elevator (0,WGrp7, 1, _V( 1.519606,.481,-7.592102), _V(-1,0,.2), (float)(-64*RAD));
 anim_ELEVATOR = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_ELEVATOR, 0, 1, &elevator);

   static UINT WGrp17[1] = {20};//right
   static MGROUP_ROTATE elevator1 (0,WGrp17, 1, _V(-1.519606,.481,-7.592102), _V(-1,0,-.2), (float)(-64*RAD));
 //anim_ELEVATOR1 = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_ELEVATOR, 0, 1, &elevator1);

static UINT RAileronGrp[1] = {33};//leftelevator
   static MGROUP_ROTATE RAileron (0,RAileronGrp, 1, _V(4.57612, 0.344, -8.212096), _V(-1,0,.52), (float)(-64*RAD));
 anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &RAileron);

   static UINT LAileronGrp[1] = {32};//right
   static MGROUP_ROTATE LAileron (0, LAileronGrp, 1, _V(-4.57612, 0.344, -8.212096), _V(-1,0,-.52), (float)(64*RAD));
 anim_laileron = CreateAnimation (0.5);
   
AddAnimationComponent (anim_laileron, 0, 1, &LAileron);





  	static UINT EGrp20[2] = {14,15};//FRONTGEAR

   	static UINT EGrp21[2] = {23,22};//frontgeaRDOOR
   	static UINT EGrp22[1] = {7};//geardoor1
   	static UINT EGrp23[1] = {4};//gear1
   	//static UINT EGrp24[1] = {33};//frtgear
   	static UINT EGrp25[5] = {11,12,13,6,5};//frtgear1
   	static UINT EGrp26[5] = {10,9,8,2,3};//frtgear2
	 //  	static UINT EGrp27[2] = {0,1};//frtgear2
   //	static UINT EGrp28[2] = {3,4};//frtgear2

    static MGROUP_ROTATE GEAR1A (0,EGrp20, 2, _V( 0,-.835,  4.698), _V(-1,0,0), (float)(-93*RAD));//frontgear
    static MGROUP_ROTATE GEAR1B (0,EGrp21, 2, _V( -.2358739, -.865,  4.689725), _V(0,0,1), (float)(-90*RAD));//frontgeardoor
    static MGROUP_ROTATE GEAR2B (0,EGrp22, 1, _V(  1.588986, .262, -1.813917), _V(0,0,1), (float)(-80*RAD));//geardoor1
      static MGROUP_ROTATE GEAR2A(0,EGrp23, 1, _V(  -1.588986, .262, -1.813917), _V(0,0,1), (float)(80*RAD));//geardoor1
      // static MGROUP_TRANSLATE GEAR3A(0,EGrp24, 1, _V(0,-.75,0));
    //static MGROUP_ROTATE GEAR3B (0,EGrp25, 5, _V( 3.969229 ,-0.197, -4.331026), _V(.45,-.3,.75), (float)(105*RAD));
	static MGROUP_ROTATE GEAR3B (0,EGrp25, 5, _V( 3.959161, .228, -4.365942), _V(0.41925,-0.2729729,0.8658574), (float)(100*RAD));
    static MGROUP_ROTATE GEAR3C (0,EGrp26, 5, _V( -3.969229 ,.228, -4.331026), _V(-0.41925,-0.2729729,0.8658574), (float)(-100*RAD));
	   // static MGROUP_ROTATE GEAR3C (0,EGrp26, 5, _V( -3.969229 ,.228, -4.331026), _V(-0.45,-0.32444,0.93), (float)(-90*RAD));

         // static MGROUP_ROTATE GEAR2C (0,EGrp27, 2, _V(  -4.002398, -0.263385, -4.369433), _V(-.2,0,1), (float)(-80*RAD));//geardoor1
      //static MGROUP_ROTATE GEAR2D(0,EGrp28, 2, _V(   4.002398, -0.263385, -4.369433), _V(-.2,0,1), (float)(-80*RAD));//geardoor1

anim_GEAR = CreateAnimation (0); 
AddAnimationComponent  (anim_GEAR, .5, 1, &GEAR1A) ; //front gear  
//anim_GEAR1B = CreateAnimation (0); 
AddAnimationComponent  (anim_GEAR, 0, .5, &GEAR1B) ; //front gear door 
//anim_GEAR2A = CreateAnimation (0); 
AddAnimationComponent  (anim_GEAR, 0, .5, &GEAR2A) ;   
//anim_GEAR2B = CreateAnimation (0); 
AddAnimationComponent  (anim_GEAR, 0, .5, &GEAR2B) ;  
//AddAnimationComponent  (anim_GEAR, 0, .5, &GEAR2C) ;  
//AddAnimationComponent  (anim_GEAR, 0, .5, &GEAR2D) ;  

//anim_GEAR3A = CreateAnimation (0); 
//AddAnimationComponent  (anim_GEAR3A, .5, 1, &GEAR3A) ;   
//anim_GEAR3B = CreateAnimation (0); 
AddAnimationComponent  (anim_GEAR, .5, 1, &GEAR3B) ;  
//anim_GEAR3C = CreateAnimation (0); 
AddAnimationComponent  (anim_GEAR, .5, 1, &GEAR3C) ;  

   	static UINT EGrp10[2] = {36,65};//gear2
    static MGROUP_ROTATE CANOPY (0,EGrp10, 2, _V( .49, .57 , 5.484), _V(0,0,1), (float)(-60*RAD));//geardoor2

	   	static UINT EGrp11[2] = {35,64};//gear2
    static MGROUP_ROTATE CANOPY1 (0,EGrp11, 2, _V( -.455, .57 , 5.484), _V(0,0,1), (float)(60*RAD));//geardoor2

anim_CANOPY1 = CreateAnimation (0); 
AddAnimationComponent  (anim_CANOPY1, 0, 1, &CANOPY) ;  
AddAnimationComponent  (anim_CANOPY1, 0, 1, &CANOPY1) ;  

   	static UINT EGrp12[2] = {68,38};//gear2
    static MGROUP_ROTATE CANOPY2 (0,EGrp12, 2, _V( .473, .618 , 4.613014), _V(0,0,1), (float)(-60*RAD));//geardoor2

	   	static UINT EGrp13[2] = {67,37};//gear2
    static MGROUP_ROTATE CANOPY3 (0,EGrp13, 2, _V( -.443, .618, 4.613014), _V(0,0,1), (float)(60*RAD));//geardoor2

anim_CANOPY2 = CreateAnimation (0); 
AddAnimationComponent  (anim_CANOPY2, 0, 1, &CANOPY2) ;  
AddAnimationComponent  (anim_CANOPY2, 0, 1, &CANOPY3) ; 


	   	static UINT EGrp100[1] = {52};//gear2
   // static MGROUP_TRANSLATE THROTTLE (0,EGrp100, 1, _V( -0.272 , .26 , 5.97), _V(1,0,-.2), (float)(135*RAD));//geardoor2
static MGROUP_TRANSLATE THROTTLE (0,EGrp100, 1, _V(0.0257819,0,0.12));
anim_THROTTLE = CreateAnimation (0); 
AddAnimationComponent  (anim_THROTTLE, 0, 1, &THROTTLE) ;  

	   	static UINT EGrp101[1] = {44};//gear2
    static MGROUP_ROTATE LGEAR (0,EGrp101, 1, _V( -0.259,  .349,  6.14), _V(1,0,0), (float)(25*RAD));//geardoor2

anim_LGEAR = CreateAnimation (0); 
AddAnimationComponent  (anim_LGEAR, 0, 1, &LGEAR) ;  

static UINT EGrp40[2] = {47,61};
    static MGROUP_ROTATE STICKZ (0,EGrp40, 2, _V(0, -.098,  5.744), _V(1,0,0), (float)(-20*RAD));//

	

	anim_STICKZ = CreateAnimation (.5);

AddAnimationComponent  (anim_STICKZ, 0, 1, &STICKZ) ;  



//RIGHTSTICK LEFT/RIGHT
static UINT EGrp41[2] = {47,61};//
static MGROUP_ROTATE STICKX (0,EGrp41, 2, _V(0,  -.098, 5.744), _V(0,0,1), (float)(-20*RAD));//


anim_STICKX = CreateAnimation (.5); 

AddAnimationComponent  (anim_STICKX, 0, 1, &STICKX) ;  


static UINT EGrp46[1] = {17};//
static MGROUP_ROTATE SPOILER1 (0,EGrp46, 1, _V(-0.8831145, -1.013791, -2.297742), _V(1,0,0), (float)(-40*RAD));//
static UINT EGrp47[1] = {18};//
static MGROUP_ROTATE SPOILER2 (0,EGrp47, 1, _V(0.8831145, -1.013791, -2.297742), _V(1,0,0), (float)(-40*RAD));//


anim_SPEEDBRAKE = CreateAnimation (0); 

AddAnimationComponent  (anim_SPEEDBRAKE, 0, 1, &SPOILER1) ;  
AddAnimationComponent  (anim_SPEEDBRAKE, 0, 1, &SPOILER2) ;  

//LEFT PEDAL

static UINT EGrp147[1] = {50};//COLL
static MGROUP_TRANSLATE PEDALL (0,EGrp147, 1, _V(0, 0,  .1));//

//static UINT EGrp52[1] = {49};//COLL
//static MGROUP_ROTATE PEDALL1 (1,EGrp52, 1, _V(0.242514, -1.753422,  5.682361), _V(1,0,0), (float)(-15*RAD));//

anim_PEDAL1 = CreateAnimation (0.5); 

AddAnimationComponent  (anim_PEDAL1, 0, 1, &PEDALL) ;  
//AddAnimationComponent  (anim_PEDAL1, 0, 1, &PEDALL1) ;  
//PEDAL RIGHT
static UINT EGrp48[1] = {49};//
static MGROUP_TRANSLATE PEDALR (0,EGrp48, 1, _V(0, 0,  -.1));//

//static UINT EGrp53[1] = {50};//VC
//static MGROUP_ROTATE PEDALR1 (1,EGrp53, 1, _V(0.242514, -1.753422,  5.682361), _V(1,0,0), (float)(15*RAD));//

anim_PEDAL2 = CreateAnimation (0.5); 

AddAnimationComponent  (anim_PEDAL2, 0, 1, &PEDALR) ;  
//AddAnimationComponent  (anim_PEDAL2, 0, 1, &PEDALR1) ;  




}
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
return new GLIDER (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
if (vessel) delete (GLIDER*)vessel;
}



// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int GLIDER::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	// only process keydown events
	if (!down) 
		return 0; 

//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	//
	// ADD REALISM: It's your responsabilities also to set ship's control accordingly to crew aboard.
	//				If you want to disable control if no one is aboard have a look at "SetADCtrlMode()"
	//				and "SetAttitudeMode()" functions of Orbiter. To disable thrusters set their fuel 
	//				ressource to NULL.
	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		// we provide feedback to user (You can display a message on panel or wathewer)
		// here below all the return code possible:
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendHudMessage(),"Transfer to docked ship Ok - %s transfered",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendHudMessage(),"EVA OK - %s left the ship",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendHudMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendHudMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendHudMessage(),"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendHudMessage(),"Error, docked ship do not use UMmu 2.0, ask author to add it");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendHudMessage(),"Misc error with UMMU install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA",
		SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),
		Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer"
			", please press \"E\" to EVA",SelectedUmmuMember,
			Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,
			Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the virtual UMMU airlock door on/off
	

	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S)
	{
		sprintf(SendHudMessage(),"%i souls aboard ship %s, %i seats available",
			Crew.GetCrewTotalNumber(),GetName(),1-Crew.GetCrewTotalNumber());
		return 1;
	}

	//---------------------------------------------------------------------------
	// ADD some Fun, Eject the guy, No check of all return code here to keep listing small and clear.
	// Notice eject function doesn't check airlock state at all.
	// GOOD IDEA: Get the Object's handle after ejection with function "GetObjHandleOfLastEVACrew"
	// and add to it one very small tank, thruster and smoke, then fire thruster (see pilot ejection of DGIV)
	// BAD IDEA: Not testing the handle returned by "GetObjHandleOfLastEVACrew" before using may
	// cause a CTD if by any bad luck the pointer is invalid (handle==NULL)
	if(key==OAPI_KEY_ESCAPE&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(Crew.EjectCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember))==EVA_OK)
			sprintf(SendHudMessage(),"%s EJECTED",Crew.GetLastEvaedCrewName());
		SelectedUmmuMember=0;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	// better idea is to use the new UMMU Id definition
	// look readme.txt in folder "config/UMMUIdConfig"
	if(key==OAPI_KEY_C)
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendHudMessage(),"Mesh changed");
		return 1;
	}

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	if(key==OAPI_KEY_M&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		AddUMmuToVessel(TRUE);
	}
	//---------------------------------------------------------------------------
	if(key==OAPI_KEY_LBRACKET)
{

  FLAP_proc = FLAP_proc+ .1;
if(FLAP_proc > 1) FLAP_proc = 1;
	
      
  return 1;
}  
if(key==OAPI_KEY_RBRACKET)
{

  FLAP_proc = FLAP_proc- .1;
if(FLAP_proc <0) FLAP_proc = 0;
	
      
  return 1;
}  


if(key==OAPI_KEY_K)
{
RevertCANOPY1();
}

if(key==OAPI_KEY_J)
{
RevertCANOPY2();
}

if(key==OAPI_KEY_G)
{
RevertGEAR();
}
if(key==OAPI_KEY_B)
{
if (speedbrake==0)
    speedbrake=1;
else 
speedbrake=0;
}
if(key==OAPI_KEY_V)
{

    CAM = CAM + 1;
    if(CAM > 1) CAM = 0;
	
        SelectCockpitView(CAM);
        return 1;
}  

	
return 0;
}

// ====================================================================
// clbkVisualCreated used to display UMMU initialisation message 
// because oapiDebugString() doesn't work in clbkSetClassCap
// ====================================================================
void GLIDER::clbkVisualCreated (VISHANDLE vis, int refcount)
{   
	MainExternalMeshVisual = GetMesh(vis,0);
	
}
// ==============================================================
// Visual destroyed
// ==============================================================
void GLIDER::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
	MainExternalMeshVisual = 0;	
}



// ========================================================================
// clbkLoadState function of Orbiter - Save scenario when user exit Orbiter 
// ========================================================================

//------------------------------------------------------------------------------------------------------------------
// This below is not mandatory for UMmu to run.
//------------------------------------------------------------------------------------------------------------------

// ==========================================================
// USING TEXT INTERFACE FOR CREW MANAGMENT (not a good idea)
//
// This is the custom functions that send oapiDebugString() Message on screen
// it simply avoid that the message is sent if our ship don't have the focus
// and ensure that message will be reminded when we focus back on ship
// the Orbiter function below clbkFocusChanged is used to erase message when
// we lost focus.
// It's a bad idea to do an UMmu user interface only by using Debug screen message 
// but if you can't do better you may use this system.
///////////////////////////////////////////////////////////////////////////////////////////

// --------------------------------------------------------------
// Orbiter's HUD callback
// used to display UMMU's message see PDF doc:
// "Example of feedback method by HUD Display"
// --------------------------------------------------------------
void GLIDER::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// UMmu display messages
	if(dHudMessageDelay>0)
	{
		TextOut (hDC,5,hps->H/60*15,cUmmuHudDisplay,strlen(cUmmuHudDisplay));
		dHudMessageDelay-=oapiGetSimStep();
		if(dHudMessageDelay<0)
			dHudMessageDelay=0;
	}
}
void GLIDER::clbkSaveState(FILEHANDLE scn)
{
char cbuf[256];
	



	// ORBITER, default vessel parameters
	SaveDefaultState (scn);


	 sprintf (cbuf, "%d %0.4f", GEAR_status, GEAR_proc);
		oapiWriteScenario_string (scn, "GEAR", cbuf);

	 sprintf (cbuf, "%d %0.4f", CANOPY1_status, CANOPY1_proc);
		oapiWriteScenario_string (scn, "CANOPY1", cbuf);

		sprintf (cbuf, "%d %0.4f", CANOPY2_status, CANOPY2_proc);
		oapiWriteScenario_string (scn, "CANOPY2", cbuf);
		
	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios(scn);
}
void GLIDER::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		

		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;


	      if (!_strnicmp (line, "GEAR", 4)) {
   sscanf (line+4, "%d%lf", &GEAR_status, &GEAR_proc);
			}
	      if (!_strnicmp (line, "CANOPY1", 7)) {
   sscanf (line+7, "%d%lf", &CANOPY1_status, &CANOPY1_proc);
			}
 if (!_strnicmp (line, "CANOPY2", 8)) {
   sscanf (line+8, "%d%lf", &CANOPY2_status, &CANOPY2_proc);
			}
		        
		// ORBITER, unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, status);
	}
SetAnimation (anim_GEAR, GEAR_proc);		

	SetAnimation (anim_CANOPY1, CANOPY1_proc);
		SetAnimation (anim_CANOPY2, CANOPY2_proc);
		
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu airlock state and set it accordingly
	// to your airlock. IE: call here "SetAirlockDoorState" TRUE of FALSE depend
	// of your animated airlock
}
bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr=(char*)data;	if(*cPtr==2){*cPtr=3;strcpy(cPtr+2,str);}
	else if(*cPtr==4){*cPtr=5;strcpy(cPtr+42,str);}
	else if(*cPtr==6){*cPtr=7;strcpy(cPtr+82,str);}return true;
}
void GLIDER::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding==FALSE&&cAddUMmuToVessel[0]==0)
		return;
	if(bStartAdding==TRUE){
		int salut=sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel,0,sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0]=1;
	}
	else if(cAddUMmuToVessel[0]==1){
		cAddUMmuToVessel[0]=2;
		oapiOpenInputBox ("Enter new crew's name (or escape)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==3){
		cAddUMmuToVessel[0]=4;
		oapiOpenInputBox ("Enter crew's age",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==5){
		cAddUMmuToVessel[0]=6;
		oapiOpenInputBox ("Enter function (Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==7){
		cAddUMmuToVessel[0]=0;
		int Age=max(5,min(100,atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2],Age,70,70,&cAddUMmuToVessel[82])==TRUE){
			sprintf(SendHudMessage(),"Crew \"%s\" aged %i added to vessel",&cAddUMmuToVessel[2],Age);
		}
		else{
			strcpy(SendHudMessage(),"ERROR: Crew not added (vessel full?)");
		}
	}
}



void GLIDER::RevertGEAR (void)
{
	GEAR_status = ((GEAR_status == GEAR_UP || GEAR_status == GEAR_RAISING) ?
		GEAR_LOWERING : GEAR_RAISING);
}

void GLIDER::RevertCANOPY1 (void)
{
	CANOPY1_status = ((CANOPY1_status == GEAR_UP || CANOPY1_status == GEAR_RAISING) ?
		GEAR_LOWERING : GEAR_RAISING);
}
void GLIDER::RevertCANOPY2 (void)
{
	CANOPY2_status = ((CANOPY2_status == GEAR_UP || CANOPY2_status == GEAR_RAISING) ?
		GEAR_LOWERING : GEAR_RAISING);
}

bool GLIDER::clbkLoadVC(int id)
{
 
  VCMFDSPEC mfd;

 //Virtual cockpit elements
 if((id>=0)&&(id<1)){
  if(id==0){
   SetCameraDefaultDirection(_V(0,0,1));
   SetCameraOffset(_V(0,.7,5.4));
   SetCameraShiftRange(_V(0,0,0.1),_V(-0.2,0,0),_V(0.2,0,0));
   oapiVCSetNeighbours(-1,-1,-1,-1);
oapiVCRegisterMFD(0,&mfd);
   mfd.ngroup=63;
   mfd.nmesh=1;
   oapiVCRegisterMFD(1,&mfd);
   
  }
 }
 return 1;
}
